-----------------------------------------------------------------------------------------------
-- Client Lua Script for LuaInvoke
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------------
-- LuaInvoke Module Definition
-----------------------------------------------------------------------------------------------
local LuaInvoke = {} 
local traceback = false
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function LuaInvoke:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	return o
end

function LuaInvoke:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
	Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- LuaInvoke OnLoad
-----------------------------------------------------------------------------------------------
function LuaInvoke:OnLoad()
	Apollo.RegisterSlashCommand("lua", "OnLuaInvokeCmd", self)
	Apollo.RegisterSlashCommand("luatrace", "OnLuaTraceInvokeCmd", self)
end

-----------------------------------------------------------------------------------------------
-- LuaInvoke Functions
-----------------------------------------------------------------------------------------------

-- on SlashCommand "/lua"
function LuaInvoke:OnLuaInvokeCmd(strCmd, strArg)
	if (strArg == "") then
		Print("-- LuaInvoke : [Command Usage]")
		Print("/lua <code>")
		Print("eg, /lua i = 1 + 1; return i")
		Print("-- LuaInvoke : [Input] i = 1 + 1; return i")
		Print("-- LuaInvoke : [Result] 2")
	else
		if (strArg == "trace") then
			LuaInvoke:OnLuaTraceInvokeCmd()
		else
			Print("-- LuaInvoke : [Input] " .. strArg)
			local func, errorMessage = loadstring(strArg)
			if (not func) then
				Print("-- LuaInvoke : [Compile Error] " .. errorMessage)
			else
				local rv = nil
				if traceback then
					rv = { xpcall(func, function(err) return debug.traceback(err) end) }
				else
					rv = { pcall(func) }
				end
				success = rv[1]
				if (not success) then
					Print("-- LuaInvoke : [Runtime Error] " .. rv[2])
				else
					local strResult = tostring(rv[2])
					for i = 3, #rv do
						strResult = strResult .. ", " .. tostring(rv[i])
					end
					Print("-- LuaInvoke : [Result] " .. strResult)
				end
			end
		end
	end
end

function LuaInvoke:OnLuaTraceInvokeCmd()
	traceback = not traceback
	Print("-- LuaInvoke : [Stack Trace] " .. tostring(traceback))
end

-----------------------------------------------------------------------------------------------
-- LuaInvoke Instance
-----------------------------------------------------------------------------------------------
local LuaInvokeInst = LuaInvoke:new()
LuaInvokeInst:Init()
